package com.ruoyi.system.service.impl;

import cn.hutool.core.util.ObjectUtil;
import com.alibaba.fastjson2.JSON;
import com.alibaba.fastjson2.JSONObject;
import com.ruoyi.common.constant.Constants;
import com.ruoyi.common.constant.ServiceConstants;
import com.ruoyi.common.core.redis.RedisCache;
import com.ruoyi.common.exception.ServiceException;
import com.ruoyi.common.pay.wx.domain.NotifyEventType;
import com.ruoyi.common.utils.DateUtils;
import com.ruoyi.common.utils.http.HttpUtils;
import com.ruoyi.system.domain.EtCallbackLog;
import com.ruoyi.system.domain.channel.ChannelVO;
import com.ruoyi.system.domain.order.RlOrder;
import com.ruoyi.system.domain.refund.RlRefund;
import com.ruoyi.system.domain.vo.AttachVo;
import com.ruoyi.system.mapper.EtCallbackLogMapper;
import com.ruoyi.system.service.*;
import com.wechat.pay.java.core.RSAAutoCertificateConfig;
import com.wechat.pay.java.core.notification.Notification;
import com.wechat.pay.java.core.notification.NotificationConfig;
import com.wechat.pay.java.core.notification.NotificationParser;
import com.wechat.pay.java.core.notification.RequestParam;
import com.wechat.pay.java.service.payments.model.Transaction;
import com.wechat.pay.java.service.refund.model.RefundNotification;
import com.wechat.pay.java.service.refund.model.Status;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.transaction.support.TransactionTemplate;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletRequest;
import java.math.BigDecimal;
import java.util.concurrent.ScheduledExecutorService;
import java.util.concurrent.TimeUnit;


/**
 * 订单支付回调 CallbackService 实现类
 */
@Service
public class CallbackServiceImpl implements CallbackService {

    private static final Logger logger = LoggerFactory.getLogger(CallbackServiceImpl.class);

    @Resource
    private IRlOrderService orderService;

    @Autowired
    private RedisCache redisCache;

    @Resource
    private IRlRefundService etRefundService;

    @Autowired
    private ScheduledExecutorService scheduledExecutorService;

    @Resource
    private EtCallbackLogMapper callbackLogMapper;

    @Autowired
    private TransactionTemplate transactionTemplate;

    @Autowired
    private RlChannelService rlChannelService;

    @Value("${aliyun.accessKeyId}")
    private String accessKeyId;

    @Value("${aliyun.accessKeySecret}")
    private String accessKeySecret;

    @Value("${aliyun.signName}")
    private String signName;

    @Value("${aliyun.templateCode}")
    private String templateCode;

    @Value("${aliyun.templateCode2}")
    private String templateCode2;

//    @Resource
//    private EtMsgLogMapper etMsgLogMapper;


    /**
     * 微信支付回调
     */
    @Override
    @Transactional
    public void weChat(HttpServletRequest request) {
        String body = HttpUtils.getBody(request);
        logger.info("【微信支付回调】接收对象（未验签） : " + JSON.toJSONString(body));
        EtCallbackLog etCallbackLog = new EtCallbackLog();
        etCallbackLog.setBody2(body);
        etCallbackLog.setType("1");
        // 解析通知数据
        Notification notification = JSON.parseObject(body, Notification.class);

        //异步保存回调日志
        asynchronousSaveCallbackLog(etCallbackLog);

        // 支付成功通知
        if (NotifyEventType.TRANSACTION_SUCCESS.getValue().equals(notification.getEventType())) {
            // 验签、解密并转换成 Transaction
            Transaction transaction = checkAndParse(request, body, Transaction.class);
            if (Transaction.TradeStateEnum.SUCCESS.equals(transaction.getTradeState())) {
                // 充值成功后的业务处理
                etCallbackLog.setBody(JSON.toJSONString(transaction));
                AttachVo attachVo = JSONObject.parseObject(transaction.getAttach(),AttachVo.class);
                businessHandle(transaction.getOutTradeNo(), attachVo, ServiceConstants.PAY_TYPE_WX);//业务处理
            }
        }
    }

    /**
     * 支付成功后的业务处理
     */
    @Override
    public void businessHandle(String outTradeNo,AttachVo attachVo,String payType) {
        // 充值成功后的业务处理
        RlOrder order = orderService.selectRlOrderByOutTradeNo(outTradeNo);
        RlOrder order1 = new RlOrder();
        order1.setOrderId(order.getOrderId());
        logger.info("【微信支付回调】订单信息 : " + JSON.toJSONString(order));
        logger.info("【微信支付回调】========== orderId : " + order.getOrderId());

        Long payChannel = order.getPayChannel();
        ChannelVO channelVO = rlChannelService.selectSmChannelByChannelId(payChannel);

//        AsUser asUser = asUserMapper.selectUserById(order.getUserId());
//        /** 支付回调逻辑  1. 处理预约还是开锁  电压 */
//        AsDevice asDevice = null;
//        if(StrUtil.isNotBlank(order.getSn())){
//            asDevice = asDeviceMapper.selectAsDeviceBySn(order.getSn());
//        }
//        if(attachVo.getType().equals(ServiceConstants.BUSINESS_TYPE_RIDING)){
//            logger.info("【微信支付回调】骑行支付");
//            // 1-骑行支付  关锁
//            EtOperatingArea area = etOperatingAreaService.selectEtOperatingAreaByAreaId(order.getAreaId());
//            order1.setMark("骑行支付");
//            logger.info("=================【微信支付回调】11111111==================");
//            if(ServiceConstants.RETURN_VERIFY_YES.equals(area.getReturnVerify())){
//                logger.info("【微信支付回调】还车-----需要-----拍照审核");
//                order1.setStatus(ServiceConstants.ORDER_STATUS_TO_BE_AUDIT);//如果还车需要拍照审核，状态为待审核
//                BigDecimal amount = order.getPayFee();
//
//                // 异步处理短信
//                if("1".equals(area.getMsgSwitch())){
//                    asynchronousMsg(order, amount);
//                }
//            }else{
//                logger.info("【微信支付回调】还车-----不需要-----拍照审核");
//                order1.setStatus(ServiceConstants.ORDER_STATUS_ORDER_END);
//                // 还车结算___小时后自动退押金---创建一个定时器TimerTask，计算出退还时间后，执行退款操作
//                logger.info("=================【微信支付回调】22222222==================");
//                // 退还押金处理
//                refundDeposit(area.getDeposit(), order, asUser);
//                logger.info("=================【微信支付回调】33333333==================");
//                // 用户付款通知
//                if("1".equals(area.getMsgSwitch())){
//                    asynchronousMsg2(order);
//                }
//            }
//            asDevice.setStatus(ServiceConstants.VEHICLE_STATUS_NORMAL);//还车后车辆正常运营
//            asDevice.setLockStatus(ServiceConstants.LOCK_STATUS_CLOSE);
//            // 新增资金流水记录
//            EtCapitalFlow capitalFlow = capitalFlowRecords(order, ServiceConstants.FLOW_TYPE_INCOME, ServiceConstants.ORDER_TYPE_RIDING, ServiceConstants.OWNER_TYPE_OPERATOR, null, ServiceConstants.PAY_TYPE_WX);
//            logger.info("=================【微信支付回调-新增资金流水记录后】=================={}",JSON.toJSON(capitalFlow));
//            order1.setHandlingCharge(capitalFlow.getHandlingCharge());
//            order1.setPlatformServiceFee(capitalFlow.getPlatformServiceFee());
//            order1.setOperatorDividend(capitalFlow.getOperatorDividend());
//            order1.setCost(getCost(channelVO,order.getPayFee()));
//            logger.info("=================【微信支付回调】4444444==================");
//        }else if(attachVo.getType().equals(ServiceConstants.BUSINESS_TYPE_APPOINTMENT)){
//            logger.info("【微信支付回调】取消预约支付");
//            // 2-取消预约支付
//            order1.setStatus(ServiceConstants.ORDER_STATUS_ORDER_END);
//            order1.setMark("取消预约支付");
//            asDevice.setStatus(ServiceConstants.VEHICLE_STATUS_NORMAL);//取消预约支付后车辆正常运营
//            asDevice.setLockStatus(ServiceConstants.LOCK_STATUS_CLOSE);
//        }else if(attachVo.getType().equals(ServiceConstants.ORDER_TYPE_COUPON)){
//            /** 优惠券订单  */
//            logger.info("【微信支付回调】优惠券支付");
//            // 3-优惠券支付
//            order1.setStatus(ServiceConstants.ORDER_STATUS_ORDER_END);
//            order1.setMark("优惠券支付");
//            // 优惠券成功处理逻辑
//            couponSuccessHandle(order);
//        }else if(attachVo.getType().equals(ServiceConstants.BUSINESS_TYPE_DEPOSIT)){
//            logger.info("【微信支付回调】押金支付");
//            // 4-押金支付
//            order1.setStatus(ServiceConstants.ORDER_STATUS_ORDER_END);
//            asUser.setBalance(order.getTotalFee());
//            order1.setMark("押金支付");
//            // 删除用户缓存
//            String token = attachVo.getToken();
//            logger.info("【微信支付回调】删除用户缓存:"+token);
//            if (StringUtils.isNotNull(token))
//            {
//                redisCache.deleteObject(CacheConstants.LOGIN_TOKEN_KEY + token);
//            }
//        }else{
//            logger.error("【微信支付回调】 : 支付场景不存在");
//            throw new ServiceException("【微信支付回调】支付场景不存在");
//        }
//        if(ObjectUtil.isNotNull(asDevice)){
//            int device = asDeviceService.updateAsDevice(asDevice);
//            if(device==0){
//                logger.error("【微信支付回调】更新车辆状态失败");
//                throw new ServiceException("【微信支付回调】更新车辆状态失败");
//            }
//        }
//
//        Boolean execute = transactionTemplate.execute(e -> {
//            order1.setPaid("1");
//            order1.setPayTime(DateUtils.getNowDate());
//            order1.setPayType(payType);
//            order1.setLocking("0");
//            // 如果使用了优惠券则扣除一次使用次数
//            if(ObjectUtil.isNotNull(order.getLogId())){
//                EtCouponUserLog couponUserLog = etCouponClaimLogMapper.selectEtCouponClaimLogByLogId(order.getLogId());
//                EtCoupon etCoupon = etCouponMapper.selectEtCouponByCouponId(couponUserLog.getCouponId());
//                logger.info("【微信支付回调】优惠券信息 : " + JSON.toJSONString(etCoupon));
//                if(ObjectUtil.isNotNull(etCoupon) && (etCoupon.getType().equals(ServiceConstants.COUPON_TYPE_DISCOUNT_CARD) || etCoupon.getType().equals(ServiceConstants.COUPON_TYPE_VOUCHER)) && couponUserLog.getLimitNum() > 0){
//                    etCouponClaimLogMapper.deductLimitNum(couponUserLog.getLogId());
//                    logger.info("【微信支付回调】优惠券使用次数-1");
//                }
//            }
//            logger.info("=================【微信支付回调】开始更新订单信息=================={}",JSON.toJSON(order1));
//            int updateEtOrder = orderService.updateEtOrder(order1);
//            if(updateEtOrder==0){
//                logger.error("【微信支付回调】更新订单信息失败");
//                throw new ServiceException("【微信支付回调】更新订单信息失败");
//            }
//            if(attachVo.getType().equals(ServiceConstants.BUSINESS_TYPE_DEPOSIT)){
//                logger.info("=================【微信支付回调】开始更新用户信息==================");
//                int updateUser = userService.updateUser(asUser);
//                if(updateUser==0){
//                    logger.error("【微信支付回调】更新用户押金失败");
//                    throw new ServiceException("【微信支付回调】更新用户押金失败");
//                }
//            }
//            logger.info("=================【微信支付回调】全部结束！！！！！==================");
//            return Boolean.TRUE;
//        });
//        if(!execute)throw new ServiceException("管理员开锁失败");
    }

    private BigDecimal getCost(ChannelVO channel,BigDecimal payFee) {
        // 获取到渠道成本
        BigDecimal costRate = channel.getCostRate();
        BigDecimal bigDecimal = costRate.divide(new BigDecimal(100), 6, BigDecimal.ROUND_HALF_UP);
        BigDecimal cost = bigDecimal.multiply(payFee).setScale(2, BigDecimal.ROUND_HALF_UP);
        logger.info("【保存资金流水记录--订单支付】 成本==============bigDecimal=====================："+bigDecimal);
        logger.info("【保存资金流水记录--订单支付】 成本==============cost=====================："+cost);
        return cost;
    }



    private void asynchronousSaveCallbackLog(EtCallbackLog etCallbackLog) {
        //开异步线程保存回调参数
        scheduledExecutorService.schedule(() -> {
            logger.info("【微信支付回调】异步保存回调参数");
            etCallbackLog.setCreateTime(DateUtils.getNowDate());
            int i = callbackLogMapper.insertEtCallbackLog(etCallbackLog);
            if(i>0){
                logger.info("【微信支付回调】异步保存回调参数成功");
            }
        }, 0, TimeUnit.SECONDS);
    }


    /**
     * 微信退款回调
     */
    @Override
    @Transactional
    public void weChatRefund(HttpServletRequest request) {
        String body = HttpUtils.getBody(request);
        EtCallbackLog etCallbackLog = new EtCallbackLog();
        etCallbackLog.setBody2(body);
        etCallbackLog.setType("2");
        logger.info("【微信退款回调】接收对象 : " + JSON.toJSONString(body));
        // 解析通知数据
        Notification notification = JSON.parseObject(body, Notification.class);
        logger.info("【微信退款回调】转换成notification: " + JSON.toJSONString(notification));
        // 退款成功通知
        if (NotifyEventType.REFUND_SUCCESS.getValue().equals(notification.getEventType())) {
            // 验签、解密并转换成 RefundNotification
            RefundNotification refundNotification = checkAndParse(request, body, RefundNotification.class);
            logger.info("【微信退款回调】转换成RefundNotification: " + JSON.toJSONString(refundNotification));
            etCallbackLog.setBody(JSON.toJSONString(refundNotification));
            if (Status.SUCCESS.equals(refundNotification.getRefundStatus())) {
                // 退款成功后  改变 etRefundService
                String outRefundNo = refundNotification.getOutRefundNo();
                RlRefund rlRefund = new RlRefund();
                rlRefund.setRefundNo(outRefundNo);
                rlRefund.setRefundResult(Constants.SUCCESS2);
                int i = etRefundService.updateEtRefundByRefundNo(rlRefund);
                if(i==0){
                    logger.error("【微信退款回调】更新退款单失败");
                    throw new ServiceException("【微信退款回调】更新退款单失败");
                }
                logger.info("【微信退款回调】全部结束！！！！！");
            }
        }
        //异步保存回调日志
        asynchronousSaveCallbackLog(etCallbackLog);
    }

    /**
     * 验签并解析
     * @param request 请求
     * @param body 请求体
     * @param clazz 返回值类型
     */
    private <T> T checkAndParse(HttpServletRequest request, String body, Class<T> clazz) {
        // 构造 RequestParam
        RequestParam requestParam = new RequestParam.Builder()
                .serialNumber(request.getHeader("Wechatpay-Serial"))
                .nonce(request.getHeader("Wechatpay-Nonce"))
                .signature(request.getHeader("Wechatpay-Signature"))
                .timestamp(request.getHeader("Wechatpay-Timestamp"))
                .body(body)
                .build();
        // 验签
        String wechatpaySerial = request.getHeader("Wechatpay-Serial");
        NotificationParser notificationParser = getNotificationParser(wechatpaySerial);
        return notificationParser.parse(requestParam, clazz);
    }


    public NotificationParser getNotificationParser(String wechatpaySerial) {
        // 根据证书序列号获取运营商
        ChannelVO channelVO = rlChannelService.selectChannelBySerialNumber(wechatpaySerial);
        if(ObjectUtil.isNull(channelVO)){
            throw new ServiceException("【微信支付回调】根据证书序列号获取支付渠道失败");
        }else{
            NotificationConfig config = new RSAAutoCertificateConfig.Builder()
                    .merchantId(channelVO.getMerchantId())
                    .privateKeyFromPath(channelVO.getPrivateKeyPath())
                    .merchantSerialNumber(channelVO.getMerchantSerialNumber())
                    .apiV3Key(channelVO.getApiV3Key())
                    .build();
            return new NotificationParser(config);
        }
    }
}
