package com.ruoyi.system.service.impl;

import cn.hutool.core.util.ObjectUtil;
import cn.hutool.core.util.StrUtil;
import com.ruoyi.common.constant.HttpStatus;
import com.ruoyi.common.constant.ServiceConstants;
import com.ruoyi.common.exception.ServiceException;
import com.ruoyi.common.utils.DateUtils;
import com.ruoyi.common.utils.bean.BeanUtils;
import com.ruoyi.common.utils.map.GeoUtils;
import com.ruoyi.common.utils.uuid.IdUtils;
import com.ruoyi.common.utils.wx.vo.PrepayResponseVO;
import com.ruoyi.system.domain.accessory.RlAccessoryVO;
import com.ruoyi.system.domain.agent.RlAgent;
import com.ruoyi.system.domain.device.RlDevice;
import com.ruoyi.system.domain.model.RlModelVO;
import com.ruoyi.system.domain.order.RlOrder;
import com.ruoyi.system.domain.order.RlOrderQuery;
import com.ruoyi.system.domain.order.RlOrderVO;
import com.ruoyi.system.domain.rule.RlFeeRule;
import com.ruoyi.system.domain.store.StoreVo;
import com.ruoyi.system.domain.vo.PriceVO;
import com.ruoyi.system.mapper.RlOrderMapper;
import com.ruoyi.system.service.*;
import com.ruoyi.system.service.store.RlStoreService;
import com.wechat.pay.java.service.payments.jsapi.model.PrepayWithRequestPaymentResponse;
import org.jetbrains.annotations.NotNull;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.math.BigDecimal;
import java.math.RoundingMode;
import java.util.*;

/**
 * 订单Service业务层处理
 *
 * @author qzz
 * @date 2024-09-19
 */
@Service
public class RlOrderServiceImpl implements IRlOrderService
{
    @Resource
    private RlOrderMapper orderMapper;

    @Resource
    private IWxPayService wxPayService;

    @Autowired
    private IRlFeeRuleService feeRuleService;

    @Resource
    private IRlCityService cityService;

    @Autowired
    private IRlAccessoryService accessoryService;

    @Autowired
    private IRlDeviceService deviceService;

    @Resource
    private IRlAgentService agentService;

    @Autowired
    private IRlModelService modelService;

    @Autowired
    private ISysConfigService configService;

    @Autowired
    private RlStoreService storeService;

    /**
     * 查询订单
     *
     * @param orderId 订单主键
     * @return 订单
     */
    @Override
    public RlOrderVO selectRlOrderByOrderId(Long orderId)
    {
        return orderMapper.selectRlOrderByOrderId(orderId);
    }

    /**
     * 查询订单
     *
     * @param orderNo 订单号
     * @return 订单
     */
    @Override
    public RlOrderVO selectRlOrderByOrderNo(String orderNo)
    {
        return orderMapper.selectRlOrderByOrderNo(orderNo);
    }

    /**
     * 第三方订单号
     *
     * @param outTradeNo 订单号
     * @return 订单
     */
    @Override
    public RlOrderVO selectRlOrderByOutTradeNo(String outTradeNo)
    {
        return orderMapper.selectRlOrderByOutTradeNo(outTradeNo);
    }


    /**
     * 查询订单
     *
     * @param orderNo 订单号
     * @return 订单
     */
    @Override
    public RlOrderVO getOrderInfoByOrderNo(String orderNo)
    {
        RlOrderVO order = orderMapper.selectRlOrderByOrderNo(orderNo);
        List<RlAccessoryVO> accessorys = accessoryService.selectRlAccessoryListByModelId(order.getModelId());
        order.setAccessorys(accessorys);
        RlDevice rlDevice = deviceService.selectDeviceBySn(order.getSn());
        if(ObjectUtil.isNotNull(rlDevice)) {
            order.setPicture(rlDevice.getPicture());
        }
        /** 超时规则*/
        Long ruleId = order.getRuleId();
        RlFeeRule feeRule = feeRuleService.selectRlFeeRuleByRuleId(ruleId);
        order.setRule(feeRule);
        return order;
    }

    /**
     * 查询订单列表
     *
     * @param rlOrder 订单
     * @return 订单
     */
    @Override
    public List<RlOrderVO> selectRlOrderList(RlOrderQuery rlOrder)
    {
        return orderMapper.selectRlOrderList(rlOrder);
    }

    /**
     * 查询所有订单列表
     *
     * @param rlOrder 订单
     * @return 订单
     */
    @Override
    public List<RlOrderVO> selectAllRlOrderList(RlOrderQuery rlOrder)
    {
        return orderMapper.selectAllRlOrderList(rlOrder);
    }

    /**
     * 新增订单
     *
     * @param rlOrder 订单
     * @return 结果
     */
    @Override
    public int insertRlOrder(RlOrderQuery rlOrder)
    {
        rlOrder.setCreateTime(DateUtils.getNowDate());
        return orderMapper.insertRlOrder(rlOrder);
    }

    /**
     * 修改订单
     *
     * @param rlOrder 订单
     * @return 结果
     */
    @Override
    public int updateRlOrder(RlOrder rlOrder)
    {
        return orderMapper.updateRlOrder(rlOrder);
    }

    /**
     * 根据订单号修改订单
     *
     * @param rlOrder 订单
     * @return 结果
     */
    @Override
    public int updateRlOrderByOrderNo(RlOrder rlOrder)
    {
        return orderMapper.updateRlOrderByOrderNo(rlOrder);
    }


    /**
     * 批量删除订单
     *
     * @param orderIds 需要删除的订单主键
     * @return 结果
     */
    @Override
    public int deleteRlOrderByOrderIds(Long[] orderIds)
    {
        return orderMapper.deleteRlOrderByOrderIds(orderIds);
    }

    /**
     * 删除订单信息
     *
     * @param orderId 订单主键
     * @return 结果
     */
    @Override
    public int deleteRlOrderByOrderId(Long orderId)
    {
        return orderMapper.deleteRlOrderByOrderId(orderId);
    }

    /**
     * 下单
     * 1. 创建订单
     * 2. 预支付
     * 3.
     *
     * @param order 订单对象
     * @return 结果
     */
    @Override
    public PrepayResponseVO createOrder(RlOrderQuery order,Long userId) {
        String orderNo = IdUtils.getOrderNo("zl");
        PrepayResponseVO responseVO = getPrepayResponseVO(order,orderNo,ServiceConstants.ORDER_TYPE_LEASE,userId);
        return responseVO;
    }

    /**
     *  确认支付
     * @param orderNo 订单对象
     * @return 结果
     */
    @Override
    public PrepayResponseVO confirmPay(String orderNo) {
        RlOrderVO order = orderMapper.selectRlOrderByOrderNo(orderNo);
        if(order.getPaid().equals(ServiceConstants.ORDER_PAY_STATUS_PAID)){
            throw new ServiceException("订单已支付");
        }
        if(!order.getStatus().equals(ServiceConstants.ORDER_STATUS_TO_BE_PAID)){
            throw new ServiceException("订单非待支付状态");
        }
        PrepayWithRequestPaymentResponse response = wxPayService.prepayWithRequestPayment(order);
        PrepayResponseVO responseVO = new PrepayResponseVO();
        responseVO.setResponse(response);
        responseVO.setOrderNo(orderNo);
        return responseVO;
    }

    @NotNull
    private PrepayResponseVO getPrepayResponseVO(RlOrderQuery order,String orderNo, String type,Long userId) {
        RlFeeRule feeRule = feeRuleService.selectRlFeeRuleByRuleId(order.getRuleId());
        setFeeRule(order, feeRule);
        /** 解析地址  analyzeAddress */
        analyzeAddress(order);
        order.setCreateTime(DateUtils.getNowDate());
        order.setOrderNo(orderNo);
        order.setPaid(ServiceConstants.ORDER_PAY_STATUS_NON_PAYMENT);
        order.setType(type);
        order.setUserId(userId);
        String configByKey = configService.selectConfigByKey("rl.order.auto.cancel");
        order.setAutoCancelTime(DateUtils.getTimeAfterXMinutes(order.getCreateTime(),Integer.parseInt(configByKey)));

        StoreVo storeVo = storeService.selectSmStoreById(order.getStoreId());
        if(ObjectUtil.isNotNull(storeVo)){
            order.setMerchantId(storeVo.getMerchantId());
        }

        /** 计算价格 */
        onceMoreCalculatePrice(order);

        order.setStatus(ServiceConstants.ORDER_STATUS_TO_BE_PAID);
        int i = orderMapper.insertRlOrder(order);
        if (i < 1) {
           throw new RuntimeException("下单失败");
        }
        PrepayWithRequestPaymentResponse response = wxPayService.prepayWithRequestPayment(order);
        PrepayResponseVO responseVO = new PrepayResponseVO();
        responseVO.setResponse(response);
        responseVO.setOrderNo(orderNo);
        return responseVO;
    }

    private void onceMoreCalculatePrice(RlOrderQuery order) {
        PriceVO priceVO = calculatePrice(order);
        order.setDeposit(priceVO.getDeposit());
        order.setLeaseFee(priceVO.getRent());
        order.setDeliveryFee(priceVO.getDeliveryFee());
        order.setTotalFee(priceVO.getTotalFee());
        order.setPayFee(priceVO.getTotalFee());
    }

    /** 解析地址 */
    private void analyzeAddress(RlOrderQuery order) {
        String address = cityService.getAddressByLocation(order.getPickupLon(),order.getPickupLat());
        order.setPickupLoc(address);
    }

    private void setFeeRule(RlOrderQuery order, RlFeeRule feeRule) {
        order.setNum(order.getNum());
        order.setPrice(feeRule.getPrice());
        order.setExplain(feeRule.getExplain());
        order.setInstructions(feeRule.getInstructions());
        order.setOutUnit(feeRule.getOutUnit());
        order.setOutPrice(feeRule.getOutPrice());
        order.setRentalUnit(feeRule.getRentalUnit());
        /** 如果是租赁订单，根据取车时间计算到期时间
         * 如果是续费订单，根据到期时间计算续费订单的到期时间 */
        Date startTime = null;
        if(order.getType().equals(ServiceConstants.ORDER_TYPE_LEASE)){
            startTime = order.getPickupTime();
        }else{
            startTime = order.getExpiryTime();
        }
        Date expiryTime = DateUtils.getTimeAfterXUnit(startTime, order.getNum(), feeRule.getRentalUnit());
        order.setExpiryTime(expiryTime);
    }

    /**
     * 扫码绑定车辆
     */
    @Override
    public int bandDevice(String orderNo, String sn) {
        RlOrderVO order = orderMapper.selectRlOrderByOrderNo(orderNo);
        if(order == null){
            throw new RuntimeException("订单不存在");
        }
        if(StrUtil.isNotBlank(order.getSn())){
            throw new RuntimeException("该订单已绑定车辆："+order.getSn());
        }
        RlOrder rlOrder = new RlOrder();
        rlOrder.setOrderNo(orderNo);
        rlOrder.setSn(sn);
        return orderMapper.updateRlOrderByOrderNo(rlOrder);
    }

    /**
     * 续租
     * 到期时间
     * 1.  根据原来的订单号，写入新的订单
     * 2.  续租订单标记类型
     * 3.
     */
    @Override
    public PrepayResponseVO relet(String orderNo, Long ruleId,Long userId) {
        if(StrUtil.isBlank(orderNo)){
            throw new RuntimeException("订单号不能为空");
        }
        RlOrderVO order = orderMapper.selectRlOrderByOrderNo(orderNo);
        RlOrderQuery orderQuery = new RlOrderQuery();
        BeanUtils.copyBeanProp(orderQuery,order);
        /** 原订单已经到期 */
        Date nowDate = DateUtils.getNowDate();
        if (order.getExpiryTime().before(nowDate)) {
            throw new RuntimeException("订单已过期，请重新下单");
        }
        orderQuery.setOriginalOrderNo(orderNo);
        orderQuery.setRuleId(ruleId);
        orderQuery.setPickupTime(order.getExpiryTime());
        String newOrderNo = IdUtils.getOrderNo("xz");
        PrepayResponseVO responseVO = getPrepayResponseVO(orderQuery,newOrderNo,ServiceConstants.ORDER_TYPE_RELET,userId);
        return responseVO;
    }

    /**
     * 订单取消
     *
     */
    @Override
    public Boolean orderCancel(String orderNo) {
        if(StrUtil.isBlank(orderNo)){
            throw new RuntimeException("订单号不能为空");
        }
        RlOrderVO order = orderMapper.selectRlOrderByOrderNo(orderNo);
        if(order == null){
            throw new RuntimeException("订单不存在");
        }
        if(!(ServiceConstants.ORDER_STATUS_TO_BE_TAKEN.equals(order.getStatus()) ||
                ServiceConstants.ORDER_STATUS_TO_BE_SENT.equals(order.getStatus()) || ServiceConstants.ORDER_STATUS_TO_BE_PAID.equals(order.getStatus()))){
            throw new RuntimeException("订单已配送或已使用，不能取消");
        }
        if(ServiceConstants.ORDER_PAY_STATUS_PAID.equals(order.getPaid())){
            throw new RuntimeException("订单已支付，不能取消");
        }
        RlOrder rlOrder = new RlOrder();
        rlOrder.setOrderNo(orderNo);
        rlOrder.setPaid(ServiceConstants.ORDER_PAY_STATUS_NON_PAYMENT);
        rlOrder.setStatus(ServiceConstants.ORDER_STATUS_CANCE);
        int i = orderMapper.updateRlOrderByOrderNo(rlOrder);
        if(i>0){
            return true;
        }
        return false;
    }

    /**
     * 车损扣款
     */
    @Override
    public Boolean deductMoney(BigDecimal money, String orderNo) {
        RlOrder rlOrder = new RlOrder();
        rlOrder.setOrderNo(orderNo);
        rlOrder.setDeductionAmount(money);
        int i = orderMapper.updateRlOrderByOrderNo(rlOrder);
        return i>0;
    }

    /**
     * 押金退款
     */
    @Override
    public Boolean depositRefund(String orderNo) {
        RlOrderVO order = orderMapper.selectRlOrderByOrderNo(orderNo);
        BigDecimal deposit = order.getDeposit();
        BigDecimal overdueFee = order.getOverdueFee();
        BigDecimal deductionAmount = order.getDeductionAmount();
        BigDecimal remainingDeposit = deposit.subtract(overdueFee).subtract(deductionAmount);
        if(remainingDeposit.compareTo(BigDecimal.ZERO) <= 0){
            return true;
        }
        if(!ServiceConstants.ORDER_PAY_STATUS_PAID.equals(order.getPaid())){
          throw new ServiceException("订单未支付，不能退款", HttpStatus.ERROR);
        }
        if(!ServiceConstants.ORDER_STATUS_ORDER_END.equals(order.getStatus())){
            throw new ServiceException("订单状态异常，不能退款", HttpStatus.ERROR);
        }
        // 执行退还剩余押金操作
        wxPayService.refund(order, "审核后退押金",remainingDeposit,order.getOutTradeNo());
        return null;
    }

    /**
     * 计算价格
     */
    @Override
    public PriceVO calculatePrice(RlOrderQuery order) {
        PriceVO priceVO = new PriceVO();
        BigDecimal totalFee = BigDecimal.ZERO;
        // 押金
        BigDecimal deposit = deposit(order.getModelId(), priceVO);
        totalFee = totalFee.add(deposit);
        // 租金
        BigDecimal rentFee = rentFee(order, priceVO);
        totalFee = totalFee.add(rentFee);
        // 配送费
        BigDecimal deliveryFee = deliveryFee(order.getAgentId(), priceVO);
        totalFee = totalFee.add(deliveryFee);
        priceVO.setTotalFee(totalFee);
        return priceVO;
    }

    /**
     * 根据订单号查询可还车的店铺
     */
    @Override
    public List<StoreVo> getReturnStoreList(String orderNo,String lon,String lat) {
        RlOrderVO order = orderMapper.selectRlOrderByOrderNo(orderNo);
        List<StoreVo> storeVos = storeService.selectSmStoreListBYMerchantId(order.getMerchantId());
        for (StoreVo storeVo:storeVos) {
            BigDecimal storeLon = storeVo.getLng();
            BigDecimal storeLat = storeVo.getLat();
            if (storeLon != null && storeLat != null && StrUtil.isNotBlank(lon) && StrUtil.isNotBlank(lat)) {
                double[] point1 = {storeLon.doubleValue(), storeLat.doubleValue()};
                double[] point2 = {Double.valueOf(lon), Double.valueOf(lat)};
                double distance = GeoUtils.haversineDistance(point1, point2);
                BigDecimal distanceFormatted = new BigDecimal(distance).setScale(1, RoundingMode.HALF_UP);
                storeVo.setDistance(distanceFormatted.doubleValue());
            }
        }
        List<StoreVo> storeList = new ArrayList<>(storeVos);
        Collections.sort(storeList, new Comparator<StoreVo>() {
            @Override
            public int compare(StoreVo o1, StoreVo o2) {
                // 注意：由于我们想要按距离由近到远排序，因此当 o1 的距离小于 o2 的距离时，应返回 -1
                // 当距离相等时返回 0，当 o1 的距离大于 o2 的距离时返回 1
                return Double.compare(o1.getDistance(), o2.getDistance());
            }
        });
        return storeList;
    }

    private BigDecimal deposit(Long modelId, PriceVO priceVO) {
        RlModelVO rlModelVO = modelService.selectEModelByModelId(modelId);
        if(rlModelVO == null){
            throw new ServiceException("车型不存在");
        }
        BigDecimal deposit = rlModelVO.getDeposit();
        if(rlModelVO == null){
            throw new ServiceException("车型不存在");
        }
        priceVO.setDeposit(deposit);
        return deposit;
    }

    private BigDecimal deliveryFee(Long agentId, PriceVO priceVO) {
        BigDecimal deliveryFee = BigDecimal.ZERO;
        RlAgent agent = agentService.selectRlAgentByAgentId(agentId);
        if(agent == null){
            throw new ServiceException("代理商不存在");
        }
        if(!agent.getIsFreeCar()){
            deliveryFee = agent.getDeliveryFee();
            priceVO.setDeliveryFee(deliveryFee);
        }
        return deliveryFee;
    }

    private BigDecimal rentFee(RlOrderQuery order, PriceVO priceVO) {
        BigDecimal rent = BigDecimal.ZERO;
        RlFeeRule feeRule = feeRuleService.selectRlFeeRuleByRuleId(order.getRuleId());
        if(feeRule == null){
            throw new ServiceException("租赁规则不存在");
        }
        Integer num = order.getNum();
        if(num != null){
            rent = feeRule.getPrice().multiply(new BigDecimal(num));
            priceVO.setRent(rent);
        }
        return rent;
    }


}
